#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: qicongsheng
import os
import re
import shutil
import urllib.request
import zipfile
from importlib.metadata import distribution
from pathlib import Path

from pystand_pack import deptree
from pystand_pack import dir_manager


def setup(**kwargs):
    output_dir = kwargs.get('output_dir', 'out')
    pystand_download_url = kwargs.get('pystand_download_url',
                                      'https://r2.nevy.eu.org/dist/pystand/PyStand-py38-pyqt5-win32-lite.zip')
    dist_dir = kwargs.get('dist_dir', 'dist')
    exe_name = kwargs.get('exe_name', 'main.exe')
    entry_point = kwargs.get('entry_point', 'main:start')
    pip_requires = kwargs.get('pip_requires', [])
    module_includes = kwargs.get('module_includes', [])
    module_excludes = kwargs.get('module_excludes', [])

    dist_path = Path(output_dir) / dist_dir
    # download pystand
    download_and_unzip(pystand_download_url, output_dir)
    # rename pystand files
    os.rename(Path(output_dir) / os.listdir(output_dir)[0], dist_path)
    os.rename(dist_path / 'PyStand.exe', dist_path / exe_name)
    os.remove(dist_path / 'PyStand.int')
    os.remove(dist_path / 'README.md')
    # copy source files to script dir
    dir_manager.copy_matching_files('./', dist_path / 'script', module_includes)
    dir_manager.delete_matching_files(dist_path / 'script', module_excludes)
    # copy dependencies
    copy_packages_to_destination(pip_requires, dist_path / 'site-packages')
    # create _pystand_static.int
    entry_point_file = entry_point.split(':')[0]
    entry_point_func = entry_point.split(':')[1]
    py_strs = [
        "import sys\n",
        "sys.path.append('./script')\n",
        f"import {entry_point_file}\n",
        f"{entry_point_file}.{entry_point_func}()\n",
    ]
    with open(dist_path / '_pystand_static.int', "w", encoding="utf-8") as file:
        file.writelines(py_strs)


def get_site_packages_path():
    """获取当前 Python 环境的 site-packages 路径"""
    # 通过查找一个已知包的路径来确定 site-packages 目录
    example_package = distribution('pip')  # 使用 pip 作为示例
    return Path(example_package.locate_file('')).absolute()


def copy_packages_to_destination(pip_requires, destination_dir):
    """根据 requirements.txt 复制包及其依赖到目标目录"""
    site_packages_path = get_site_packages_path()
    pip_requires = set(pip_requires)
    destination_path = Path(destination_dir)
    destination_path.mkdir(parents=True, exist_ok=True)  # 创建目标目录
    all_dependencies = deptree.get_package_dependencies(pip_requires)
    # 复制包及其依赖
    for package in all_dependencies:
        package = re.sub(r"[<>=!].+", "", package)
        package_path = Path(site_packages_path) / package
        if package_path.exists():
            dest_path = destination_path / package
            if dest_path.exists():
                shutil.rmtree(dest_path)  # 如果目标目录已存在，先删除
            shutil.copytree(package_path, dest_path)
            print(f"Copied: {package}")
        else:
            print(f"Skipped: {package} (not found in site-packages)")


def download_and_unzip(url, extract_to):
    try:
        if os.path.exists(extract_to):
            shutil.rmtree(extract_to)  # 递归删除文件夹
        os.makedirs(extract_to)
        zip_path = os.path.join(extract_to, 'temp.zip')
        urllib.request.urlretrieve(url, zip_path)
        with zipfile.ZipFile(zip_path, 'r') as zip_ref:
            zip_ref.extractall(extract_to)
        os.remove(zip_path)
    except Exception as e:
        print(f"发生错误: {e}")
